#include "Arduino.h"
#include <SPI.h>

#include "config.h"
#include "CommandHandler.h"
#include "DisplayProxyMAX7456.h"
#include "max7456.h"

Max7456 osd;
DisplayProxyMAX7456 display(&osd);
CommandHandler cmd_handler(&display);

/**
 * Setup handler (Arduino Style)
 * 
 * Write some debug information to the serial port and initialize the display.
 */
void setup()
{
  Serial.begin(BAUDRATE);

  Serial.println("================================================================================");
  Serial.println("Firmware: " PROJECT_NAME);
  Serial.println("Version:  " VERSION_STRING);
  Serial.println("Built:    " __DATE__ ", " __TIME__);
  Serial.println("================================================================================");
  Serial.println("Initialize...");
  SPI.begin();
  display.setOffset(DISP_OFFSET_X, DISP_OFFSET_Y);

  display.write(0, 0, "==========================");
  display.write(0, 1, "Firmware: " PROJECT_NAME);
  display.write(0, 2, "Version:  " VERSION_STRING);
  display.write(0, 3, "Built:    " __DATE__ ", " __TIME__);
  display.write(0, 4, "==========================");

  delay(3000);
  display.clear();
  Serial.println("Ready!");
}

/**
 * Loop handler (Arduino Style)
 * 
 * Continously read the serial port if input is available. On line change send to the command handler.
 */
void loop()
{
  while (Serial.available() > 0)
  {
    String input = Serial.readStringUntil('\n');
    cmd_handler.parseJSON(input.c_str());
  }
}